const path = require('path')
const fs = require('fs')
const sh = require('shelljs')

const parse = require('../parser/parse')
const { hashFilename } = require('../parser/hasher')
const { state } = require('../state')


module.exports = async () => {

  // console.log(state.deps)
  
  const src = state.config.srcRoot
  const dest = state.config.destRoot
  
  let files = []
  files.push(... state.entries.filter(o => !path.basename(o).startsWith('_')))
  // files.push(... state.entries.filter(o => o !== state.config.layout))
  files.push(... state.deps.filter(o => {
    const ext = o.path.split('.').pop()
    // if (o.path.startsWith('http')) return false
    if (!state.proc.bundle) return true
    if (!['css', 'js'].includes(ext)) return true
    if (o.ref.endsWith('.html')) return true
    return false
  }).map(o => o.path))
    
  // console.log('****', files)
  sh.rm('-rf', dest)
  
  await Promise.all(
    files.map(async filename => {
      const ext = filename.split('.').pop()
      // console.log('**', filename)
      // if (filename.startsWith('http')) return
      // const destFilename = path.join(dest, filename)
      let destFilename = path.join(dest, path.relative(src,  filename))
      if (state.proc.hash && !filename.endsWith('.html')) {
        const hash = await hashFilename(filename)
        destFilename = destFilename.split('.').map((o, i, c) => i === c.length - 2 ? o + '-' + hash : o).join('.')
      }
      // console.log('**DEST', destFilename)
      // return
      
      sh.mkdir('-p', path.dirname(destFilename))
      const dep = state.deps.find(o => o.path === filename)
      const { content } = await parse(filename, 'prod', dep && dep.type)
      // const content = await loader(src + '/' + filename, 'prod')
      let cpErr = null
      if (content && ['html', 'css', 'js'].includes(ext)) {
        fs.writeFileSync(destFilename, content)
      } else {
        const cpRes = sh.cp(filename, destFilename)
        if (cpRes.stderr) cpErr = `Cannot find ${filename} referenced by ${state.deps.find(o => o.path === filename).ref}`
        }
        if (cpErr) console.error(cpErr)
          
          
          // console.log('copy', filename, destFilename, await hash(filename))
      try {
      } catch (error) {
        console.log('%%', filename, src, error.message)
      }
    })
  )

  if (state.proc.transpile) {
    const babelPolyfill = fs.readFileSync(require.resolve('babel-polyfill/dist/polyfill.min.js'), 'utf8')
    fs.writeFileSync(path.join(path.join(dest, path.relative(src, state.clientRoot)), 'babel-polyfill.js'), babelPolyfill)
  }

  if (state.entryType === 'express') {
    const serverRoot = path.dirname(state.serverRoot)
    sh.mkdir('-p', path.join(dest, path.relative(src,  serverRoot)))
    sh.cp('-r', serverRoot + '/*', path.join(dest, path.relative(src,  serverRoot)) + '/')
    sh.cp('package.json', dest)
    sh.cp('package-lock.json', dest)
  }

  process.exit()
}
